#include "Camera.h"
#include "math/Matrix.h"
#include "math/Vector.h"
#include <math.h>
#include <stdlib.h>

#define PI_OVER_360 0.0174532925

Camera _camera;

Camera* Camera_Init(
	float x, float y, float z,
	float lookX, float lookY, float lookZ,
	float norX, float norY, float norZ,
	float zNear, float zFar,
	float fov, double ratio)
{
	_camera.eye.x = x;
	_camera.eye.y = y;
	_camera.eye.z = z;
	_camera.center.x = lookX;
	_camera.center.y = lookY;
	_camera.center.z = lookZ;
	_camera.up.x = norX;
	_camera.up.y = norY;
	_camera.up.z = norZ;
	_camera.zNear = zNear;
	_camera.zFar = zFar;
	_camera.fov = fov;
	_camera.ratio = ratio;

	return &_camera;
}

Camera* Camera_Get(void)
{
	return &_camera;
}

void Camera_ToProjectionMatrix(float *ret)
{
	float f = 1.0 / tan(_camera.fov * PI_OVER_360);

	Matrix_Identity4m(ret);

	/*
									
		| ret0  ret1  ret2  ret3  |
		| ret4  ret5  ret6  ret7  |
		| ret8  ret9  ret10 ret11 |
		| ret12 ret13 ret14 ret15 |

	*/

	ret[0] = f / _camera.ratio;
	ret[5] = f;
	ret[10] = (_camera.zFar + _camera.zNear) / (_camera.zNear - _camera.zFar);
	ret[11] = (2.0f * _camera.zFar * _camera.zNear) / (_camera.zNear - _camera.zFar);
	ret[14] = -1.;
	ret[15] = 0.;
}

void Camera_ToTransformMatrix(float* ret)
{
	Vector3f dir = Vector_Sub3v(&_camera.center, &_camera.eye);
	Vector3f right = Vector_Cross3v(&dir, &_camera.up);
	Vector3f up = Vector_Cross3v(&right, &dir);

	dir = Vector_Normalize3v(&dir);
	right = Vector_Normalize3v(&right);
	up = Vector_Normalize3v(&up);

	Matrix_Identity4m(ret);

	/*
									
		| ret0  ret1  ret2  ret3  |
		| ret4  ret5  ret6  ret7  |
		| ret8  ret9  ret10 ret11 |
		| ret12 ret13 ret14 ret15 |

	*/

	ret[0] = right.x;
	ret[1] = right.y;
	ret[2] = right.z;

	ret[4] = up.x;	
	ret[5] = up.y;
	ret[6] = up.z;	

	ret[8] = -dir.x;
	ret[9] = -dir.y;
	ret[10] = -dir.z;

	ret[12] = -Vector_Dot3v(&right, &_camera.eye);
	ret[13] = -Vector_Dot3v(&up, &_camera.eye);
	ret[14] = -Vector_Dot3v(&dir, &_camera.eye);
}