#include "Matrix.h"
#include <stdlib.h>

float* Matrix_Identity4m(float *r)
{
	r[0] = r[5] = r[10] = r[15] = 1;
	r[1]=r[2]=r[3]=r[4]=r[6]=r[7]=r[8]=r[9]=r[11]=r[12]=r[13]=r[14]=r[15]=0;
	return r;
}

float* Matrix_Mult4mm(const float* a, const float *b, float *r)
{
	/*

							  | b0  b1  b2  b3  |
							  | b4  b5  b6  b7  |
							  | b8  b9  b10 b11 |
							  | b12 b13 b14 b15 |

		| a0  a1  a2  a3  |	  | r0  r1  r2  r3  |
		| a4  a5  a6  a7  |	  | r4  r5  r6  r7  |
		| a8  a9  a10 a11 |	  | r8  r9  r10 r11 |
		| a12 a13 a14 a15 |   | r12 r13 r14 r15 |
	*/

	r[0] = a[0]*b[0]+a[1]*b[4]+a[2]*b[8]+a[3]*b[12];
	r[1] = a[0]*b[1]+a[1]*b[5]+a[2]*b[9]+a[3]*b[13];
	r[2] = a[0]*b[2]+a[1]*b[6]+a[2]*b[10]+a[3]*b[14];
	r[3] = a[0]*b[3]+a[1]*b[7]+a[2]*b[11]+a[3]*b[15];

	r[4] = a[4]*b[0]+a[5]*b[4]+a[6]*b[8]+a[7]*b[12];
	r[5] = a[4]*b[1]+a[5]*b[5]+a[6]*b[9]+a[7]*b[13];
	r[6] = a[4]*b[2]+a[5]*b[6]+a[6]*b[10]+a[7]*b[14];
	r[7] = a[4]*b[3]+a[5]*b[7]+a[6]*b[11]+a[7]*b[15];

	r[8] = a[8]*b[0]+a[9]*b[4]+a[10]*b[8]+a[11]*b[12];
	r[9] = a[8]*b[1]+a[9]*b[5]+a[10]*b[9]+a[11]*b[13];
	r[10] = a[8]*b[2]+a[9]*b[6]+a[10]*b[10]+a[11]*b[14];
	r[11] = a[8]*b[3]+a[9]*b[7]+a[10]*b[11]+a[11]*b[15];

	r[12] = a[12]*b[0]+a[13]*b[4]+a[14]*b[8]+a[15]*b[12];
	r[13] = a[12]*b[1]+a[13]*b[5]+a[14]*b[9]+a[15]*b[13];
	r[14] = a[12]*b[2]+a[13]*b[6]+a[14]*b[10]+a[15]*b[14];
	r[15] = a[12]*b[3]+a[13]*b[7]+a[14]*b[11]+a[15]*b[15];

	return r;
}

float* Matrix_Mult4mv(const float* a, const float *b, float *r)
{
	/*

							  | b0 |
							  | b1 |
							  | b2 |
							  | b3 |

		| a0  a1  a2  a3  |	  | r0 |
		| a4  a5  a6  a7  |	  | r1 |
		| a8  a9  a10 a11 |	  | r2 |
		| a12 a13 a14 a15 |   | r3 |
	*/

	r[0] = a[0]*b[0]+a[1]*b[1]+a[2]*b[2]+a[3]*b[3];
	r[1] = a[4]*b[0]+a[5]*b[1]+a[6]*b[2]+a[7]*b[3];
	r[2] = a[8]*b[0]+a[9]*b[1]+a[10]*b[2]+a[11]*b[3];
	r[3] = a[12]*b[0]+a[13]*b[1]+a[14]*b[2]+a[15]*b[3];

	return r;
}